import types

from matplotlib.lines import lineMarkers
import matplotlib.numerix as nx

from compClust.mlx import labelings
from compClust.mlx.views import RowPCAView
from compClust.util import unique
from compClust.iplot.views import IPlotView

class Mapper:

  """
  Base class for PlotView Mapper Classes.
  """

  def __init__(self, plotView):
    """
    All the various mappers operate on an instance of IPlotView, 
    """
    if not isinstance(plotView, IPlotView):
      raise ValueError("Need instance of IPlotView, received %s instead"%
                       (str(plotView)))
    self.__plotView = plotView

  def getPlotView(self):
    return(self.__plotView)

class IDataMapper(Mapper):

  """
  This is a support class for the IPlotView class which descrbes
  how a dataset is projected onto the graph
  """

  def getPlotData(self):

    """
    plotData(self)
    
    returns a list of X,Y tuple Tuples.

    example: 
    [ ((x1,x2,x3,x4), (y1,y2,y3,y4))
      ((x1,x2,x3,x4), (y1,y2,y3,y4)) ]
    
    """
    raise NotImplementedError()
  
  def getLineTrace(self):
    """
    getLineStyle(self)
    
    retuns how the points when be connected...  If None, then the BLT
    default line connectors are used.

    Must be:
      'increasing', 'decreasing', 'both' or  None
    """

    return(None)
  
  def getWeightsAndSytles(self):

    """
    getWeights (self)

    This allows for marker control on a datapoint by datapoint
    basis...  If you want control on an element by element basis use
    getPlotMarkes and getMarkerSize functions inside of the
    MarkersMappers.
    
    returns either None or a list of tuples equal in length to the
    plotData returns value where the tuple is the weights tuple and
    the cooresponding styles tuple (see the BLT documentation for more
    information (this is very powerful)
    """

    return(None)

class ColorMapper(Mapper):

  """
  This is a support class for the IPlotView class which descrbes
  how the elements in the view should be colored.  
  """

  def getPlotColors(self):
    """

    getColors(self)

    returns:
      None for monocolor plots (all elements will be drawn in the same color)
      or
      List of HSV tuples with length = numRows
      ie [(.1,1,1), (.2, 1,1), ...]

    see RowColorMapper for more details. 
    """

    return (None)

class MarkersMapper(Mapper):

  """
  This is a support class for the IPlotView class which descrbes
  how the element markers and connecting lines should be plotted.

  """

  def getMarkerColors(self):
    """
    """

    return(None)

  def getPlotMarkers(self):
    """
    getPlotMarkers

    returns a list of markers to be added to the plot...  These are
    overriden by the weights/styles """

    return(None)

  def getMarkerSizes(self):
    """
    getMarkerSize

    this function returns the desired marker size.  These are
    overriden by the weights/styes.  """

    return( 6.0 )
  
class AnnotationMapper(Mapper):

  """
  This is a support class for the IPlotView class which descrbes
  how the elements should be annotated.
  """

  
  def getPrimaryLabeling(self):

    """
    getPrimaryAnnoations(self)

    returns a None or a labeling to be used as the primary element
    annotations - element names... these must each be unique.
    
    """

    return(None)

  def getSecondaryLabeling(self):

    """
    getSecondaryAnnoations(self)

    returns a None or a labeling to be used as the secondary element
    annotations - perhaps long discriptions
    
    """
    return(None)

class BindingsMapper(Mapper):

  def addBindings(self):

    """
    addBindings(self)

    This function if implimented adds novel event bindings to the
    plotView after the plot is generated.  The IPlot graph class has a
    few default bindings which will be left, unless overriden.

    """

    pass
    
###
#
# Implimentations of the Above Interfaces.
#
########

class SimpleBindingsMapper(BindingsMapper):

  """
  Provides some convient acces to RowAnnotations
  """

  def addBindings(self):

    self.__plot = self.getPlotView()._getPlot()
    self.__annotationMapper = self.getPlotView().getAnnotationMapper()
    self.__plot.element_bind('all', '<Button-1>', self.displayAnnotations)
    self.__activeElements = {}

  def displayAnnotations(self, event):

    """
    displayAnnotations(self, event)

    Overides the IPlot display information to display both the primary
    and secondary information in the upper right hand corner.

    """

    closest_widget = event.widget.element_closest(event.x, event.y)
    if closest_widget is None:
      return
    index = closest_widget['index']
    x = event.widget.element_closest (event.x, event.y)['x']
    y = event.widget.element_closest (event.x, event.y)['y']
    primaryName   = event.widget.element_closest (event.x, event.y)['name'].split('__')[0]

    primaryLabeling   = self.__annotationMapper.getPrimaryLabeling()
    secondaryLabeling = self.__annotationMapper.getSecondaryLabeling()
    if primaryLabeling is None:
      row = int(primaryName)
    else:
      row = primaryLabeling.getRowsByLabel(primaryName)[0]
    if secondaryLabeling is not None:
      try:
        secondaryName = reduce(lambda x,y: str(x)+str(y), secondaryLabeling.getLabelsByRow(row))
      except:
        pass
    else:
      secondaryName = ''

    status = "%s: (%3.2f, %3.2f) -- %s "%(primaryName.strip(), x, y, secondaryName.strip())

    coords = (self.__plot.xaxis_limits()[0], self.__plot.yaxis_limits()[1])
    self.__plot.marker_configure('selection', coords= coords, background="lightblue", text=status, anchor='w', under=0, hide=0)
    #self.__plot.marker_bind('selection', '<Control-Button-1>', lambda ev, row=row: self.displayLabelings(ev, row))
    #self.__plot.marker_bind('selection', '<Control-Button-1>', lambda ev, row=row: SummaryWindow(self.getPlotView(), row, parent=self.__plot.master))
    self.__plot.marker_bind('selection', '<Control-Button-1>', lambda ev, row=row: SummaryWindow(self.getPlotView(), row))    
    self.__plot.marker_bind('selection', '<Button-1>', self.hideSelection)

  def hideSelection(self, event):

    """
    hideDisplay(self, event)
    """

    # this hack makes the labels go away before a plot referesh
    self.__plot.marker_configure('selection',  coord=(-100000,-1000000))
    self.__plot.marker_configure('selection', under=1, hide=1)

  def displayLabelings(self, event, row):

    """
    displayLabelings(self, event)

    This little guy pops up a dialog with all the labelings attached
    to the dataset and what that point has been labeled as
    """

    ds = self.getPlotView().getDataset()
    primaryLabeling = self.__annotationMapper.getPrimaryLabeling()
    if primaryLabeling is None:
      name = str(row)
    else:
      name = primaryLabeling.getLabelsByRow(row)[0]

    # set up a new scrolled window for the display
    win = Tkinter.Toplevel()
    sf = Pmw.ScrolledFrame(win,
                           labelpos = 'n', label_text = 'Labels for %s'%(name),
                           usehullsize = 1,
                           hull_width = 700,
                           hull_height = 500,
                           horizflex= 'expand',
                           vertflex = 'expand'
                           )
    sf.pack(fill='both', expand=1)
    frame = sf.interior()
    gridRow = 0
    for labeling in ds.getLabelings():
      tkLabel = Tkinter.Label(frame, text=str(labeling), anchor='e')
      tkLabel.grid(row = gridRow, col = 0)
      labels = labeling.getLabelsByRow(row)
      gridCol = 1
      for label in labels:
        # this block is a little dirty, but attaches the button state to each button and makes sure that
        # the callback is notified of the current button state.
        v = Tkinter.IntVar()
        button = Tkinter.Checkbutton(frame,
                                     text=str(label),
                                     anchor='e',
                                     variable = v)
        button.var = v
        button.configure(command=lambda labeling=labeling, label=label, state=button.var:
                         self.activateMembers(labeling, label, state))

        button.grid(row=gridRow, column= gridCol)
        gridCol +=1
      gridRow +=1

  def activateMembers(self, labeling, label, state):

    """
    activateMembers(self, labeling, label, button)

    using the BLT selection feature, toggle the selection of all
    elements which are marked by the label in labeling.
    """

    plot = self.getPlotView()._getPlot()
    rows = labeling.getRowsByLabel(label)
    primaryLabeling   = self.__annotationMapper.getPrimaryLabeling()
    if primaryLabeling is None:
      elements = rows
    else:
      elements = map(lambda x:primaryLabeling.getLabelsByRow(x)[0], rows)

    if state.get() == 1:
      for element in elements:
        plot.element_activate(element)
        if self.__activeElements.has_key(element):
          self.__activeElements[element] += 1
        else:
          self.__activeElements[element] = 1

    else:
      for element in elements:
        if self.__activeElements.get(element) <= 1:
          plot.element_deactivate(element)
          del(self.__activeElements[element])
        else:
          self.__activeElements[element] -=  1         
        
class RowAnnotationMapper(AnnotationMapper):

  """
  provides a mapping of the primary and secondary annotations to
  labelings.

  """

  def __init__(self, plotView, labeling1 = None, labeling2 = None):

    """
    __init__(self, labeling1 = None, labeling2 = None)

    """
    AnnotationMapper.__init__(self, plotView)
    self.__lab1 = None
    self.__lab2 = None
    if labeling1:
      #labeling = labelings.castToGlobalLabeling(plotView.getDataset(), labeling1)
      self.__lab1 = labelings.castToGlobalWrapper(labeling1, dataset=plotView.getDataset(), removeLocal=False)
      #labelings.GlobalWrapper(plotView.getDataset(), glabeling=labeling1)
    if labeling2:
      #labeling = labelings.castToGlobalLabeling(plotView.getDataset(), labeling2)
      self.__lab2 = labelings.castToGlobalWrapper(labeling2, dataset=plotView.getDataset(), removeLocal=False)
      #labelings.GlobalWrapper(plotView.getDataset(), glabeling=labeling2)

  def getPrimaryLabeling(self):

    """
    getPrimaryAnnoations(self)

    returns a None or a labeling to be used as the primary element
    annotations - element names... these must each be unique.
    
    """

    return(self.__lab1)

  def getSecondaryLabeling(self):

    """
    getSecondaryAnnoations(self)

    returns a None or a labeling to be used as the secondary element
    annotations - perhaps long discriptions
    
    """

    return(self.__lab2)

  def setPrimaryLabeling(self, labeling):

    """
    setPrimaryLabeling(self, labeling):
    
    This labeling must be unique.
    """

    self.__lab1 = labeling

  def setSecondaryLabeling(self, labeling):

    """
    setSecondaryLabeling(self, labeling):
    
    """

    self.__lab2 = labeling

class RowMarkersMapper(MarkersMapper):

  symbols = {"plus": '+', "square": 's',  "cross": '+', "triangle": '^',  "circle": 'o'}
  def __init__(self, plotView):
    MarkersMapper.__init__(self, plotView)
    self.dataset = self.getPlotView().getDataset()
    self.__maxSize = 10
    self.__minSize = 1
    self.__sizes   = 5.0
    self.__markers = '+'

  def setPlotMarkers(self, marker='o'):
    if lineMarkers.has_key(marker):
      self.__markers = [marker]
    elif RowMarkersMapper.symbols.has_key(marker):
      self.__markers = [symbols[marker]]
    else:
      print "unrecognized marker"
      
  def setMaxSize(self, max):
    self.__maxSize = max
    if type(self.__sizes) in [types.ListType, types.TupleType, nx.ArrayType]:
      self.__sizes = scaleList(self.__sizes, minReturn=self.__minSize, maxReturn = self.__maxSize)
    else:
      if self.__sizes > self.__maxSize:
        self.__sizes = self.__maxSize
    
  def setMinSize(self, min):
    self.__minSize = min
    if type(self.__sizes) in [types.ListType, types.TupleType, nx.ArrayType]:
      self.__sizes = scaleList(self.__sizes, minReturn=self.__minSize, maxReturn = self.__maxSize)
    else:
      if self.__sizes < self.__minSize:
        self.__sizes = self.__minSize
    
  def getMarkerSizes(self):
    """
    getMarkerSize

    this function returns the desired marker size.  These are
    overriden by the weights/styes.  """

    return(self.__sizes)
  
  def setMarkerSizeByLabeling(self, labeling, minValue=None, maxValue=None):
    self.__sizes =  scaleList(labeling.getLabelByRows(), minValue, maxValue, self.__minSize, self.__maxSize)
    
  def setMarkerSizeByColValue(self, col, minValue=None, maxValue=None):
    data = self.dataset.getColData(col)
    if maxValue is None:
      maxValue = nx.mlab.max(data)
    if minValue is None:
      minValue = nx.mlab.min(data)
    self.__sizes= scaleList(data, minValue, maxValue, self.__minSize, self.__maxSize)

  def setMarkerSizeByIndex(self, minValue=None, maxValue=None):
    self.__sizes = scaleList(range(self.dataset.getNumRows()), minValue, maxValue, self.__minSize, self.__maxSize)

  def setMarkerSizeByFunction(self,function, minValue=None, maxValue=None):
    """
    sets the maker size by the given function...  the function prototype should be:
    value = function(dataset, row)
    """
    data = map(lambda row: function(self.dataset, row), range(self.dataset.getNumRows()))
    self.__sizes = scaleList(data, minValue, maxValue, self.__minSize, self.__maxSize)

  def invertMarkerSizes(self):

    if type(self.__sizes) in [types.ListType, nx.ArrayType, types.TupleType]:
      numRows = len(self.__sizes)
      max = nx.mlab.max(self.__sizes)
      min = nx.mlab.min(self.__sizes)
      self.__sizes = map(operator.sub, [max]*numRows, self.__sizes)

  def setUniformMarkerSize(self, size= .04):
    self.__sizes = size

  def clearSizes(self):
    self.__sizes = None
  
  def getPlotMarkers(self):
    """
    getPlotMarkers

    returns a list of markers to be added to the plot...  These are
    overriden by the weights/styles """

    return(self.__markers)

  def setPlotMarkersByLabeling(self, labeling, labels=None):

    """ sets the plot markers according to labeling.  If optional
    labels are supplied the marker shapes set only for the specified
    labels.  Note that there are only 8 unique marker styles, so if
    more than 8 labels are listed (or contained inside of labeling)
    symbols will cycle.
    """

    numSymbols = len(RowMarkersMapper.symbols)
    numRows = self.dataset.getNumRows()

    if labels is None:
      labels = listOps.unique(labeling.getLabelByRows())

    if len(labels) > numSymbols:
      symbols = RowMarkersMapper.symbols*int(nx.ceil(len(labels)/numSymbols))
    else:
      symbols = RowMarkersMapper.symbols
    
    labelToSymbol = {}
    map(labelToSymbol.setdefault, labels, symbols[:len(labels)])

    self.__markers = map(labelToSymbol.get, labeling.getLabelByRows(), [RowMarkersMapper.symbols[-1]]*numRows)
    return(labelToSymbol)

  def setUniformPlotMarker(self, marker='circle'):

    """
    sets the marker symbol to be the same for all points.

     are the availble shapes
     'plus', 'square',  'diamond',  'cross', 'splus', 'scross', 'triangle',  'circle' and 'none'
    """
    self.__markers = marker

class RowColorMapper(ColorMapper):

  """

  This Class provides a very flexible dynamically adjustable color
  mapper.  Several methods are supplied to set colors by, in essence
  HSV color parameters can be mapped to several attributes of a
  dataset/labeling

  """

  def __init__(self, plotView):

    """
    This constructor does little more than call IPlotView's constructor
    """

    ColorMapper.__init__(self, plotView)
    self.dataset = self.getPlotView().getDataset()
    
    # these are the HSV lists that are used to color plots with.  
    self.__h = None
    self.__s = None
    self.__v = None

  def setColorByLabelingCounts(self, labeling, component='h', minValue=None, maxValue=None, colorRange=(0,.7)):

    """ sets the given color component's value proportional to the
    size of the class
    """

    sizeDict = {}
    map(lambda x: sizeDict.setdefault(x, len(labeling.getRowsByLabel(x))),  labeling.getLabels())
    values = map(sizeDict.get,  map(labeling.getLabelByRow, range(self.dataset.getNumRows())), [0]*self.dataset.getNumRows())
    values = scaleList(values, minValue, maxValue, colorRange[0], colorRange[1])
    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values
                               
  def setColorByLabeling(self, labeling, labels=None, component='h', minValue=None, maxValue=None, colorRange=(0,.7)):
  
    """
    setColorByLabeling(self, labeling=None, component='h', minValue=None, maxValue=None, colorRange(0,.7))

    uses the labeling to produces a color list for plotting.  If the
    labeling contains real values, then the optional parameters
    minValue and maxValue can be used to scale the colorRange used.
    Only uses the first label attached to every row

    """
    if labels is None:
      rowLabels = labeling.getLabelByRows()
    else:
      tmp = {}
      map(tmp.setdefault, labels, labels)
      rowLabels = map(tmp.get, labeling.getLabelByRows(), ['__background__']*self.dataset.getNumRows())

    values = scaleList(rowLabels, minValue, maxValue, colorRange[0], colorRange[1])    

    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values

  def setColorByTupleLabeling(self, labeling, element=0, component='h', minValue=None, maxValue=None, colorRange=(0,.7)):
  
    """
    setColorByTupleLabeling(self, labeling=None, component='h', minValue=None, maxValue=None, colorRange(0,.7))

    uses the labeling to produces a color list for plotting.  If the
    labeling contains real values, then the optional parameters
    minValue and maxValue can be used to scale the colorRange used.
    Only uses the first label attached to every row

    """
    values = scaleList(map(lambda x:x[element], labeling.getLabelByRows()), minValue, maxValue, colorRange[0], colorRange[1])

    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values
      
  def setColorByColValue(self, col, component = 'h', minValue=None, maxValue=None, colorRange=(0,.7)):
    
    """
    setHueByColValue(self,col, hRange(0,1))

    sets the color to be a function of the value for that row at a given column.
    """

    data = self.dataset.getColData(col)
    if maxValue is None:
      maxValue = nx.mlab.max(data)
    if minValue is None:
      minValue = nx.mlab.min(data)

    values = scaleList(data, minValue, maxValue, colorRange[0], colorRange[1])
    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values

  def setColorByFunction(self, function, component='h',  minValue=None, maxValue=None, colorRange=(0,.7)):

    """
    setColorByFunction(self, funct, component='h', minValue, maxValue, colorRange(0,.7))

    function should take in a dataset and a row and return a single
    float.  Colors will be scaled based on that value.
    
    function prototype should be:

    value = function(dataset, row)
    """
    
    data = map(lambda row: function(self.dataset, row), range(self.dataset.getNumRows()))
    values = scaleList(data, minValue, maxValue, colorRange[0], colorRange[1])
    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values
    
  def setColorByIndex(self, component = 'h', minValue=None, maxValue = None, colorRange=(0,.7)):

    """
    setColorByFunction(self, funct)

    your function needs to return a valid HSV tuple for each row of
    the dataset, given the dataset.
    ie.

    [listOfColorTuples] = function(ds)

    """
    values = scaleList(range(self.dataset.getNumRows()), minValue, maxValue, colorRange[0], colorRange[1])
    if component   == 'h':
      self.__h = values
    elif component == 's':
      self.__s = values
    elif component == 'v':
      self.__v = values
  
  def setColors(self, list):

    """
    setColors(self, list)

    This takes in a list of HSV tuples and sets them to be the colors
    for plotting.

    """

    self.clearColors()
    self.__h = []
    self.__s = []
    self.__v = []
    for h,s,v in list:
      self.__h.append(h)
      self.__s.append(s)
      self.__v.append(v)

  def clearColors (self):
    """
    clearColors(self)

    clear set plot coloring.
    """
    self.__h = None
    self.__s = None
    self.__v = None

  def getPlotColors(self):
    """
    getPlotColors(self)

    An implimentation of the only required colorMapper class.

    """

    if self.__h == None:
      self.__h = [0] * self.dataset.getNumRows() 
    if self.__s == None:
      self.__s = [1] * self.dataset.getNumRows() 
    if self.__v == None:
      self.__v = [1] * self.dataset.getNumRows() 

    return(zip(self.__h, self.__s, self.__v))

class ProbabilityColorMapper(ColorMapper):

  """
  A fixed probabiliy color mapper
  """

  def __init__(self, plotView, probabilities):

    """
    
    """

    ColorMapper.__init__(self, plotView)
    self.dataset = self.getPlotView().getDataset()
    self.setProbabilityLabeling(probabilities)
    self.setHueByPartitioning()
    
    # these are the HSV lists that are used to color plots with.  

  def setProbabilityLabeling(self, probabilities):
      
    self.probabilities = probabilities

  def setHueByPartitioning(self, colorRange=(0,.7)):

    """
    setHueByPartioning.

    Colors each point's hue by the most likely cluster, and then it's
    value (intensity) is specified by the probability of membership to
    that particular cluster
    """
    
    self.__h = scaleList(map(nx.argmax, self.probabilities.getLabelByRows()), minReturn=colorRange[0], maxReturn=colorRange[1])
    self.__s = scaleList(map(lambda p: p[nx.argmax(p)] ,self.probabilities.getLabelByRows()), minReturn=0, maxReturn=1)
    self.__v = None

  def setHueByProbability(self, cluster, colorRange=(0,.7)):
      
    """
    setHueByProbability(self, cluster, colorRange(0,1)):
    
    where cluster is the index of the probability tuple
      
    """
    self.__h = scaleList(map(lambda x: x[int(cluster)],  self.probabilities.getLabelByRows()), minReturn=colorRange[0], maxReturn=colorRange[1])
    self.__s = None
    self.__v = None
    
  def getPlotColors(self):
    
    if self.__h == None:
      self.__h = map(lambda x: 0, range(0,self.dataset.getNumRows()))
    if self.__s == None:
      self.__s = map(lambda x: 1, range(0,self.dataset.getNumRows()))    
    if self.__v == None:
      self.__v = map(lambda x: 1, range(0,self.dataset.getNumRows()))

    return(zip(self.__h, self.__s, self.__v))
    
class RowDataMapper(IDataMapper):
  
  """
  RowDataMapper(Mapper):

  A simple data mapper which plots each row of the dataset has a line
  and seperate element.

  """

  def __init__(self, plotView, xAxisLabeling=None):

    IDataMapper.__init__(self, plotView)
    self.dataset = plotView.getDataset()
    self.setXAxisLabeling(xAxisLabeling)

  def setXAxisLabeling(self, labeling=None):
    """
    setXAxisLabeling(self, labeling=None)

    sets the xvalues for the plot.  If no labeling is specified then
    the x-axis values are set to be the index. 
    """

    if labeling is not None:
      self.__xvalues = tuple(labeling.getLabelByCols())
    else:
      self.__xvalues = tuple(range(0, self.dataset.getNumCols()))

  def getXValues(self):
    return(self.__xvalues)

  def getPlotData(self):

    ds = self.dataset
    data = zip((self.getXValues(),) * ds.getNumRows(), map(tuple, ds.getData()))
    return(data)

class SortedRowDataMapper(RowDataMapper):
  """
  SortedRowDataMapper(Mapper):

  datamapper which plots each row according to a sort-order specified
  """
  def __init__(self, plotView, xAxisLabeling=None):
    IDataMapper.__init__(self, plotView)
    self.dataset = plotView.getDataset()
    self.setXAxisLabeling(xAxisLabeling)
    self.sortedDataset = SortedView(self.dataset)

  def setXAxisLabeling(self, labeling=None):
    """
    sets xAxislabeling to be the globalWrapper labeling for the unsortedDataset
    """

    self.xAxisLabeling = labeling

  def sortByLabeling(self, labeling=None):
    """
    sets column-labeling on which sorting is performed
    """

    self.sortLabeling=labeling
    labels = labeling.getLabels()
    sortedCols = []
    for label in labels:
      sortedCols += self.sortLabeling.getColsByLabel(label)
    
    self.sortedDataset.permuteCols(sortedCols)

  def getPlotData(self):
    ds = self.sortedDataset
    xl = self.xAxisLabeling
    if xl:
      labeling = labelings.GlobalWrapper(ds, glabeling=self.xAxisLabeling.g)
      self.__xvalues = tuple(labeling.getLabelByCols())      
    else:
      labeling = None
      self.__xvalues = tuple(range(0, ds.getNumCols()))
      
    data = []
    for row in range(ds.getNumRows()):
      data.append((self.__xvalues, tuple(ds.getRowData(row))))

    return data

class MeanDataMapper(RowDataMapper, AnnotationMapper, BindingsMapper):

  """
  MeanStdDataMapper provides a view to look at the mean and std of
  either the whole datasets of groupings as specified by a labeling
  """

  def __init__(self, plotView, labeling=None):

    RowDataMapper.__init__(self, plotView)
    self.dataset = plotView.getDataset()
    self.__showStd = 0
    self.setLabeling(labeling)

  def setLabeling(self, labeling=None):
    """
    setLabeling(self, labeling)

    set the labeling to use when partitioning the dataset.
    """

    if labeling is None:
      if self.dataset.getLabeling('__meanDataMapper__'):
        labeling = self.dataset.getLabeling('__meanDataMapper__')
      else:
        labeling = Labeling(self.dataset, '__meanDataMapper__')
        labeling.labelRows  ([0]*self.dataset.getNumRows())
      self.__meanDataset = RowAggregateFunctionView(self.dataset, labeling, nx.mlab.mean)
      self.__stdDataset  = RowAggregateFunctionView(self.dataset, labeling, safeStdDev)
    else:
      self.__meanDataset.setLabeling(labeling)
      self.__stdDataset.setLabeling(labeling)

    self.dataset = self.__meanDataset

  def getLineTrace(self):
    return('increasing')

  def getPlotData(self):
    ds = self.dataset
    if self.__showStd:
      meanData = self.__meanDataset.getData()
      stdData = self.__stdDataset.getData()
      data = zip([self.getXValues()]*3, map(tuple, nx.concatenate(((meanData - stdData ) ,
                                                                        meanData,
                                                                        (meanData+stdData)))))
    else:
      data = zip((self.getXValues(),) * ds.getNumRows(), map(tuple, ds.getData()))
      
    return(data)
    
  def showStdDev(self, state=None):

    """
    showStdDev(self, state)
    
    """
    if state == None:
      self.__showStd = not self.__showStd
    else:
      self.__showStd = state

class ColumnScatterDataMapper(IDataMapper):

  """
  ColumnScatterPlotView plots two columns of a dataset against
  each other.
  """

  def __init__(self, plotView, xCol=0, yCol=1):

    IDataMapper.__init__(self, plotView)

    self.dataset = plotView.getDataset()
    self.setXColumn(xCol)
    self.setYColumn(yCol)
      
  def getPlotData(self):
    return(zip(self.dataset.getColData(self.__xCol), self.dataset.getColData(self.__yCol)))

  def getXColumn(self):
    """
    returns the column of the date being plotting as X-data
    """
    return(self.__xCol)
    
  def getYColumn(self):
    """
    returns the column of the data being plotting as Y-data 
    """
    return(self.__yCol)

  def setXColumn(self, col):
    """
    setXColumn(self, row)

    sets row to be the column of the data to be plotting as X-data
    """
    self.__xCol = col
    
  def setYColumn(self, col):
    """
    setXColumn(self, col)
    
    sets row to be the column of the data to be plotting as X-data
    """
    self.__yCol = col


class PCADataMapper(IDataMapper):

  """
  ColumnScatterPlotView plots two columns of a dataset against
  each other.
  """

  def __init__(self, plotView, xCol=0, yCol=1):

    IDataMapper.__init__(self, plotView)

    self.dataset = plotView.getDataset()
    self.dataset.addViewDefault('RowPCAView', RowPCAView, plotView.getDataset())
    self.setXColumn(xCol)
    self.setYColumn(yCol)
      
  def getPlotData(self):
    return(zip(self.dataset.getColData(self.__xCol), self.dataset.getColData(self.__yCol)))

  def getXColumn(self):
    return(self.__xCol)

  def getYColumn(self):
    return(self.__yCol)


  def setXColumn(self, col):
    """
    setXColumn(self, row)

    sets row to be the column of the data to be plotting as X-data
    """
    self.__xCol = col
    
  def setYColumn(self, col):
    """
    setXColumn(self, col)
    
    sets row to be the column of the data to be plotting as X-data
    """
    self.__yCol = col


def scaleList(values, minValue=None, maxValue=None, minReturn=0, maxReturn=1):

  """
  scaleList(values, minReturn=0, maxReturn=1, minValue=None, maxValue=None)

  Given a list of objects, returns a list of values returns a list of
  values scalled between 0 and 1 where minValue -> 0 and maxValue ->
  1).  If values is non-numeric each unique item is mapped arbitrarly
  into the 0..1 range.  Values must be a list of hashable python
  objects

  """

  labels =  unique.unique (values)
  numericLabels = 1
  for label in labels:
    labelType = type(label)
    if labelType not in [types.IntType, types.LongType, types.FloatType]:
      numericLabels = 0
      break

  if numericLabels:
    try:
      labels = nx.array(labels)
      data = nx.array(values)
    except:
      numericLabels = 0

  if numericLabels:
    if minValue is None:
      minValue = nx.mlab.min(labels)
    if maxValue is None:
      maxValue = nx.mlab.max(labels)
  else:
    minValue = 0
    maxValue = len(labels)
    labelMap = {}
    for label, count in zip(labels, range(maxValue)):
      labelMap[label]= count
    data  = map(lambda x: labelMap[x], values)

  # perhaps there is a more elegent way of doing this... But this
  # works to scale the data between minValue and maxVale
  data = nx.array(data)
  data = nx.array(map(lambda x: max(x,0), data - minValue))
  data = data.astype('f')
  data = data / ((maxValue-minValue)+.0001)
  data = nx.array(map(lambda x: min(x, 1), data))
  
  # thie maps the data into the minReturn..maxReturn range.
  data = (data * (maxReturn - minReturn)) + minReturn

  return(data)
