###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2005 by:                                                 #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
###########################################################################
#
#       Authors: Joe Roden, Brandon King & Diane Trout
# Last Modified: $Date: 2005/01/26 00:00:00 $
#      Revision: $$
#
import copy
import math
import sets

import matplotlib
import matplotlib.numerix as nx
try:
  import matplotlib.pylab as pylab
except ImportError, e:
  # try older version of matplotlib matlab(r)-like commands
  import matplotlib.matlab as pylab

# ttest from "stats.py" in pyNMS, so requires PYTHONPATH to ~/Software/pyNMS/lib
# see also python-stats package
#import stats

from rpy import *

from compClust.mlx import datasets
from compClust.mlx import labelings
from compClust.mlx import views
from compClust.mlx.views import RowPCAView
from compClust.score.ConfusionMatrix2 import ConfusionMatrix

#matplotlib.use('TkAgg')

class pcaGinsu:
  # NOTE, if you change the parameter list here, you'll need to change it in the
  # compClust.iplot.IPlot* modules, compClust.mlx.pcaGinsu, and PCAGinsu
  def __init__(self, dataset, nOutliers=10, sigCutoff = 0.05, maxPCNum = None,
               verbose = False, rowPCAView = None, makeLabelings = True):
    """
    Description: 

    Creates a pcaGinsu object given a dataset and optional parameters that
    control the pcaGinsu analysis.

    The object holds the dataset, constructs (or is given) a RowPcaView on
    which subsequent analysis is based, and parameters, all the stuff that is
    necessary for a pcaGinsu analysis.

    Simplest Usage:
      p = pcaGinsu ( myDataset )
      p.generateResults ( ['ProbeId','Description'], ['Samples'] )

    Optional Arguments:
    - nOutliers:     This is the number of data points that will be labeled
                     at both extremes of each principal component.  Default=10
    - sigCutoff:     This is the significance level below which you reject the
                     hypothesis that the high outliers and low outliers are
                     drawn from the same distribution.  Wilcoxon test determines
                     the likelihood of this hypothesis for each condition, and
                     the conditions that meet this threshold are labeled 
                     "up" or "down" conditions.  Default = 0.05.
    - maxPCNum:      If you specify this, you can limit the number of principal
                     components to fewer than the number of columns.  This is
                     useful if you have a very high dimensional dataset and are
                     not interested in analyzing every last principal component.
                     The number is 1-origin, so if you say 3 you will produce
                     labelings for PC1, PC2 and PC3.  Default = the number of
                     columns in given dataset.
    - verbose:       If set to True, will output status messages to standard
                     output while sub-operations are taking place.
                     Default = False.
    - rowPCAView:    You can provide a RowPCAView for the existing dataset so 
                     a new rowPCAView will not be created, but the existing one 
                     will be used.  By default (none given) one will be created 
                     (unless there's already one named "Row PCA View" attached 
                     to the dataset).
    - makeLabelings: If False, the row & column labelings that pcaGinsu
                     normally creates at construction will not be performed.
                     This is useful to save computation & space in case you are
                     passing a dataset that already has the proper pca row and 
                     column labelings to the constructor.  More recently the
                     code can figure out if the labelings already exist and
                     avoid re-creating the labelings.
    Notes/ideas:
    - Assumes it's given a MLX dataset.  (should verify & error if not, or
      maybe create one !)
    - maxPCNum is passed as 1-origin, and is also stored internally as
      1-origin so that it matches nCols, this can be used directly in range.
    - Regarding makeLabelings, row labelings names are unique to the number
      of outliers, but column labelings are *not*; we should make them unique
      so that one is permitted explore different nOutliers without throwing
      away previously computed column labelings.
    - We want to make the significances and mean outlier differences per
      column into column labelings so one routine computes them, and the
      other routines that need them just reference them. I've started this 
      by writing makeColumnLabelingForPCNOutlierMeanDiffs, but the other
      routines don't look up the resulting column labeling yet.  Consider
      doing the same for signfiicances as well.  Make sure the labeling
      names contain the number of outliers so exploring multiple nOutliers is
      easy & efficient.
    - Probably want to make makeRowLabelingsForPCOutliers loop over a function
      that computes the row labeling for one principal component (like 
      makeColumnLabelingsForSigGroups does), so that we can explore just a
      few principal components quickly without computing all of them.
    """
    self.verbose = verbose
    if self.verbose:
      print "Initializing pcaGinsu:"
    self.dataset = dataset
    dData = self.dataset.getData()
    self.maxOriginalData = max(max(dData))
    self.minOriginalData = min(min(dData))
    
    if rowPCAView is not None:
      if self.verbose:
        print "  1. using existing PCA View..."
      self.rowPCAView = rowPCAView
    else:
      # still we can check if one is already attached to this dataset
      self.rowPCAView = self.dataset.addViewDefault('RowPCAView', RowPCAView, self.dataset)

    self.nOutliers = nOutliers
    self.sigCutoff = sigCutoff
    self.nRows = self.rowPCAView.getNumRows()
    self.nCols = self.rowPCAView.getNumCols()

    if maxPCNum is not None:
      self.maxPCNum = maxPCNum
    else:
      self.maxPCNum = self.nCols

    if makeLabelings:
      if self.verbose:
        print "  2. setting up row labelings for outliers..."
      self.makeRowLabelingsForPCOutliers()

      if self.verbose:
        print "  3. setting up column labelings for significant groups..."
      self.makeColumnLabelingsForSigGroups()

    if self.verbose:
      print "  Done initializing pcaGinsu."

  def getOutputForPCNOutliers(self, pcNum, labelingNameList=[]):
    """
    Returns a 2D string array describing each of the high & low outliers
    for the given pcNum.  The rows contain first the high then the low
    outliers (one row per outlier), and the columns contain the values from 
    the labelings in the labelingNameList (one column per row labeling name
    provided, e.g. a 'ProbeID' column, a 'Description' column, etc.).
    etc.  The very first row contains column headings (could make this
    optional in the future, controlled by a flag).  The very first column
    contains the value of the "PC-<N> <nOutliers> High/Low" labeling, 
    (either "high" or "low").  (Again, could be optional & controlled by a
    flag in the future).

    You can use write2DStringArrayToFile if you want this output to a file.
    
    :Parameters:
      -`pcNum`: which principal compoent to return
      -`labelingNameList`: the list of row labeling names (or labels) to return
                           results for
    """
    labelingName = 'PC-%d %d High/Low' % (pcNum, self.nOutliers)
    labeling = self.dataset.getLabeling(labelingName)
    if labeling is None:
      self.makeRowLabelingsForPCOutliers()
      labeling = self.dataset.getLabeling(labelingName)

    #Output (list of lists)
    outputDict = {}

    #Get all indices for this labeling
    indices = []
#    for label in labeling.getAllRowLabels():
#      keys = labeling.getKeysByLabel(label)
#      if len(keys) > 1:
#        msg = 'FIXME: Do not know what to do with more than one key per label'
#        raise ValueError, msg
#      indices.extend(keys)
    highRows = labeling.getRowsByLabel('high')
    lowRows  = labeling.getRowsByLabel('low')
    indices.extend(highRows)
    indices.extend(lowRows)

    #Prep outputDict
    outputDict['header'] = []
    for index in indices:
      outputDict[index] = []  

    #Process each label in labelingNameList
    for labelName in labelingNameList:
      # if we're a labeling don't bother trying to do a lookup 
      if isinstance(labelName, labelings.Labeling):
        rowLabeling = labelName
      else:
        rowLabeling = self.dataset.getLabeling(labelName)
        if rowLabeling is None:
          raise ValueError("<%s> was not found in the labels attached to <%s>" % \
                           (labelName, self.dataset.getName()))
      rowLabels = rowLabeling.getAllRowLabels()
# JCR removed code because it doesn't work, not sure what the goal was
#      numOfColLabels = reduce(lambda x,y: x+y, map(lambda x: len(x), col))

      #Skip labeling if numOfColLabels == 0
#      if numOfColLabels == 0:
#        continue

      #Add header for column
      outputDict['header'].append(labelName)

      #Get label for this labeling for each of the labels from this
      # PC dimension.
      for index in indices:
        label = rowLabeling.getLabelByKey(index)
        outputDict[index].append(label)

    #Prep final output
    output = []

    #Prepare header
    header = []
    header.append(rowLabeling.getName())
    header.append('PC-%d Value' % pcNum)
    header.extend(outputDict['header'])
#    output.append(header)  wait till after sorting to "append"

    #Prepare data
    for index in indices:
      row = []
      row.append(rowLabeling.getLabelByRow(index))
      pcaRowData = self.rowPCAView.getData(index)
      row.append(pcaRowData[pcNum-1])
      row.extend(outputDict[index])
      output.append(row)

    # sort output by 2nd column, the PC-N value for each row
    output.sort(secondColumnCompare)
    output.insert(0,header)

    return output

  def getOutputForSigGroups(self, pcNum, labelingNameList=[]):
    """
    Returns a 2D string array describing each of the columns of the dataset.
    Rows of the output describe each column in the dataset, (one row per
    dataset column) and the output rows are sorted by decreasing difference
    of high means and low means, i.e. the value
      mean(high outliers) - mean(low outliers)
    computed per column of the dataset.  The columns of the output contain
    annotations describing the dataset columns, one column per column
    labeling name provided.  The first row is a header line (could be
    optional). The first column is the value of the
    "PC-<N> <M>-outlier Up/Flat/Down" labeling, either "up", "flat" or "down".
    (could also be optional).

    To make interpretation of results easiest, this output row ordering is
    meant to be the same as the plot X axis ordering of the appropriate
    outlier-trajectory plot, namely plotPCNOutlierRowsInSigGroupOrder
    Possibly need to make two (plot & output) routines or options to them
    that control the ordering method.
    
    Note: This output rows are approximately sorted as up/flat/down, because
    softing by decreasing mean difference approximates that.  In the future 
    we might want/need to produce output for all "up" first, then all "flat", 
    then all "down", and within group sort by decreasing mean difference.

    You can use write2DStringArrayToFile if you want this output to a file.
    
    :Parameters:
      -`pcNum`: which principal compoent to return
      -`labelingNameList`: the list of column labeling names (or labels) to return
                           results for
    """
    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    sigGroupLabeling = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                                % (pcNum, self.nOutliers))
    upcols   = sigGroupLabeling.getColsByLabel('up')
    flatcols = sigGroupLabeling.getColsByLabel('flat')
    downcols = sigGroupLabeling.getColsByLabel('down')

    data = self.dataset.getData()
    
    highdata = pylab.zeros((len(highrows),self.nCols),'d')
    for i in range(0,len(highrows)):
      highdata[i,:] = data[highrows[i],:]
    
    lowdata = pylab.zeros((len(lowrows),self.nCols),'d')
    for i in range(0,len(lowrows)):
      lowdata[i,:] = data[lowrows[i],:]
    
    # Average down columns, to get average for each condition
    lowMeans  = nx.average(lowdata, 0)
    highMeans = nx.average(highdata,0)
    
    meanDiffs = nx.subtract(highMeans,lowMeans);
    
    meansort = pylab.sort(meanDiffs,0)
    # want to output columns in descending order by mean differences
    meansort = pylab.fliplr([pylab.sort(meansort)])[0]

    sortedIndices = []

    for i in range(0,self.nCols):
      currcond = pylab.find(meanDiffs == meansort[i])
      index = currcond[0]
      sortedIndices.append(index)

    #Output (list of lists)
    outputDict = {}

    #Prep outputDict
    outputDict['header'] = []
    for index in sortedIndices:
      outputDict[index] = []  

    #Process each label in labelingNameList
    for labelName in labelingNameList:
      # if we're a labeling don't bother trying to do a lookup 
      if isinstance(labelName, labelings.Labeling):
        colLabel = labelName
      else:
        colLabeling = self.dataset.getLabeling(labelName)

      #Add header for column
      outputDict['header'].append(labelName)

      #Get label for this labeling for each of the labels from this
      # PC dimension.
      for index in sortedIndices:
        label = colLabeling.getLabelByCol(index)
        outputDict[index].append(label)

    #Prep final output
    output = []

    #Prepare header
    header = []
    header.append(sigGroupLabeling.getName())
    header.extend(outputDict['header'])
    output.append(header)

    #Prepare data
    for index in sortedIndices:
      row = []
      row.append(sigGroupLabeling.getLabelByCol(index))
      row.extend(outputDict[index])
      output.append(row)

    return output


  def makeRowLabelingsForPCOutliers(self):
    """
    Creates all row labelings (or 1 thru maxPCNum) sequentially that
    describe the high & low outliers for each principal component.
    The number of outliers is controlled by the pcaGinsu object's
    nOutliers value, which is typically set by the pcaGinsu
    constructor.
    """
    for pcaDim in range(self.maxPCNum):

      labelingName = 'PC-%d %d High/Low' % (pcaDim+1, self.nOutliers)
      if self.dataset.getLabeling(labelingName) is not None:
        if self.verbose:
          print '     - Row labeling %s already exists; skipping' % labelingName
        continue
      if self.verbose:
        print '     - Creating row labeling: %s' % labelingName

      pcaData = self.rowPCAView.getData()
      pcaCol = [ line[pcaDim] for line in pcaData ]
      pcaCol.sort()
  
      lowRows  = pcaCol[:self.nOutliers]
      highRows = pcaCol[-self.nOutliers:]
  
      lowIndices  = [ i for i in range(self.nRows) \
                      if pcaData[i][pcaDim] <= max(lowRows)  ]
      highIndices = [ i for i in range(self.nRows) \
                      if pcaData[i][pcaDim] >= min(highRows) ]
  
      label = labelings.GlobalLabeling(self.rowPCAView, labelingName)
      label.addLabelToRows(self.rowPCAView, 'high', highIndices)
      label.addLabelToRows(self.rowPCAView, 'low',  lowIndices)

  def makeColumnLabelingsForSigGroups(self):
    """
    Call makeColumnLabelingForPCNSigGroups once for each principal component.
    In the future, might be nice to make this work with an optional input
    range.
    """
    for pcaDim in range(self.maxPCNum):
      self.makeColumnLabelingForPCNSigGroups(pcaDim+1)

  def makeColumnLabelingForPCNSigGroups(self,pcNum):
    """
    Create one labeling of the columns given a principal component number 
    (1-based, e.g. pc 1, pc 2, ...).  A new column labeling named
    'PC-<N> <M>-outlier Up/Flat/Down' assign values 'up','flat', and 'down'
    to each column as follows:

      up   = the PC-N high outliers for this column are significantly higher
             than the PC-N low outliers

      flat = the PC-N high outliers for this column are not significantly
             different than PC-N low outliers

      down = the PC-N high outliers for this column are significantly lower
             than PC-N low outliers
    """
    newLabelingName = 'PC-%d %d-outlier Up/Flat/Down Columns' % \
                      (pcNum, self.nOutliers)
    if self.dataset.getLabeling(newLabelingName) is not None:
      if self.verbose:
        print '     - Column labeling %s already exists; skipping' \
              % newLabelingName
      return
    if self.verbose:
      print '     - Creating column labeling: %s' % newLabelingName

    outlierLabelingName = 'PC-%d %d High/Low' % (pcNum, self.nOutliers)
    l = self.dataset.getLabeling(outlierLabelingName)
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    data = self.dataset.getData()
    cnos = range(1,self.nCols+1)
    
    highdata = pylab.zeros((len(highrows),self.nCols),'d')
    for i in range(0,len(highrows)):
      highdata[i,:] = data[highrows[i],:]
    
    lowdata = pylab.zeros((len(lowrows),self.nCols),'d')
    for i in range(0,len(lowrows)):
      lowdata[i,:] = data[lowrows[i],:]
    
    ttestData = pylab.zeros(self.nCols,'d')
    for i in range(0,self.nCols):
#      junk,ttestData[i] = stats.ttest_rel(lowdata[:,i],highdata[:,i])
      rResult = r.wilcox_test(lowdata[:,i],highdata[:,i])
      ttestData[i] = rResult['p.value']
    
    # This is one approach, but doesn't break the conditions up into groups
    ttsort = pylab.sort(ttestData,0)
    lowsort  = pylab.zeros((len(lowrows), self.nCols),'d')
    highsort = pylab.zeros((len(highrows),self.nCols),'d')
    for i in range(0,self.nCols):
      currcond = pylab.find(ttestData == ttsort[i])
      index = currcond[0]
      lowsort[:,i]  = lowdata[:,index]
      highsort[:,i] = highdata[:,index]
    
    # Average down columns, to get average for each condition
    lowMeans  = nx.average(lowdata, 0)
    highMeans = nx.average(highdata,0)
    
    meanDiffs = nx.subtract(highMeans,lowMeans);
    
    ups   = pylab.find(meanDiffs >= 0)
    downs = pylab.find(meanDiffs <  0)
    sigs  = pylab.find(ttestData <= 0.05)
    
    upSig   = pylab.find(nx.logical_and((meanDiffs >= 0),(ttestData <= 0.05)))
    downSig = pylab.find(nx.logical_and((meanDiffs <  0),(ttestData <= 0.05)))
    flat    = pylab.find(ttestData > 0.05)

    label = labelings.GlobalLabeling(self.rowPCAView, newLabelingName)
    label.addLabelToCols(self.rowPCAView, 'up',    upSig)
    label.addLabelToCols(self.rowPCAView, 'flat',  flat)
    label.addLabelToCols(self.rowPCAView, 'down',  downSig)
 
  def makeColumnLabelingForPCNOutlierMeanDiffs(self, pcNum):
    """
    Creates a labeling for this dataset that contains the mean difference 
    of high outliers vs. low outliers, i.e. calculates and saves the value 
      mean(high outliers) - mean(low outliers)
    Into a labeling named "PC-<N> <M>-outlier Mean Differences".  

    Note: In a fit of refactoring I got the idea that it is smarter
    to calculate this info once and attach it as a labeling so that other 
    pcaGinsu functions can just look up this info, rather than compute these
    values again.  However, the other routines have not been changed to call
    this yet.  *** ALSO, should do the same for t-test significance!
    """
    newLabelingName = 'PC-%d %d-outlier Mean Differences' % \
                      (pcNum, self.nOutliers)
    if self.dataset.getLabeling(newLabelingName) is not None:
      if self.verbose:
        print '     - Column labeling %s already exists; skipping' \
              % (newLabelingName)
      return
    if self.verbose:
      print '     - Creating column labeling: %s' % newLabelingName


    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    c = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                 % (pcNum,self.nOutliers))
    upcols   = c.getColsByLabel('up')
    flatcols = c.getColsByLabel('flat')
    downcols = c.getColsByLabel('down')

    data = self.dataset.getData()
    
    highdata = pylab.zeros((len(highrows),self.nCols),'d')
    for i in range(len(highrows)):
      highdata[i,:] = data[highrows[i],:]
    
    lowdata = pylab.zeros((len(lowrows),self.nCols),'d')
    for i in range(len(lowrows)):
      lowdata[i,:] = data[lowrows[i],:]
    
    # Average down columns, to get average for each condition
    lowMeans  = nx.average(lowdata, 0)
    highMeans = nx.average(highdata,0)
    
    meanDiffs = nx.subtract(highMeans,lowMeans);

    label = labelings.GlobalLabeling(self.dataset, newLabelingName)
    for i in range(self.nCols):
      label.addLabelToCol(self.rowPCAView, meanDiffs[i], i)


  def scoreColumnLabelingsForPCN(self,pcNum, verbose=False):
    """
    For each column labeling (except the Up/Flat/Down labelings) compute 
    the NAMI value for that column labeling's partitioning vs. this pcNum's
    implied Up/Flat/Down column partitioning.
    """
    ufd = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                   % (pcNum,self.nOutliers))
    if ufd == []:
      msg = 'ERROR: no Up/Flat/Down labeling for PC number ' + pcNum
      raise ValueError, msg

    scores = []
    columnLabelings = []
    allLabelings = self.dataset.getLabelings()
    for l in allLabelings:
      if l.getName().find('Up/Flat/Down') == -1 and l.isColLabeling():
        columnLabelings.append(l)

    for l in columnLabelings:
      if l.isNumeric():

        print 'column ' + l.getName() + ' is numeric'
        upcols   = ufd.getColsByLabel('up')
        flatcols = ufd.getColsByLabel('flat')
        downcols = ufd.getColsByLabel('down')

        upVsFlat   = 1.0;
        upVsDown   = 1.0;
        flatVsDown = 1.0;

        # Warning, 2 argument version assumes all labels are GlobalLabelings
        if len(upcols)>0:
#          upvals   = l.getLabelsByCols(self.rowPCAView,upcols)
          upvals   = l.getLabelsByCols(upcols)
          upvals   = map(lambda x: x[0], upvals)
        if len(flatcols)>0:
#          flatvals = l.getLabelsByCols(self.rowPCAView,flatcols)
          flatvals = l.getLabelsByCols(flatcols)
          flatvals = map(lambda x: x[0], flatvals)
        if len(downcols)>0:
#          downvals = l.getLabelsByCols(self.rowPCAView,downcols)
          downvals = l.getLabelsByCols(downcols)
          downvals = map(lambda x: x[0], downvals)

#         if len(upvals)>0:
#           upranks   = stats.rankdata(upvals);
#         if len(flatvals)>0:
#           flatranks = stats.rankdata(flatvals);
#         if len(downvals)>0:
#           downranks = stats.rankdata(downvals);

        if len(upcols)>0 and len(flatcols)>0:
#          tval,pval  = stats.ttest_ind(upvals,flatvals)
#          upVsFlat   = pval
          rResult  = r.wilcox_test(upvals,flatvals)
          upVsFlat   = rResult['p.value']
#          print upvals  DEBUGGING
#          print flatvals
#          print pval
        if len(upcols)>0 and len(downcols)>0:
#          tval,pval  = stats.ttest_ind(upvals,downvals)
#          upVsDown   = pval
          tval,pval  = r.wilcox_test(upvals,downvals)
          upVsDown   = rResult['p.value']
        if len(flatcols)>0 and len(downcols)>0:
#          tval,pval  = stats.ttest_ind(flatvals,downvals)
#          flatVsDown = pval
          rResult  = r.wilcox_test(flatvals,downvals)
          flatVsDown = rResult['p.value']
        if verbose:
          print '3 scores are: '+str(upVsFlat) + ',' + str(upVsDown) + \
                ',' + str(flatVsDown)
        scores.append(min([upVsFlat,upVsDown,flatVsDown]))

      else:
        if verbose:
          print 'column ' + l.getName() + ' is discrete'
        cm = ConfusionMatrix([ufd,l])
        scores.append(cm.averageNMI())
      
    return columnLabelings, scores

  
class pcaGinsuVisualizeMatplotlib(pcaGinsu):
  """
  pcaGinsuVisualizeMatplotlib uses pcaGinsu to construct all the necessary
  labelings and then constructs non-interactive matplotlib plots showing the
  various outlier representations.  The generateResults function calls most of
  the other output plot functions herein to dump all a complete set of results
  to files, and so is useful in a batch mode to get all PCA results for data
  analysis, or to generate graphics for publication.
  """
  def plotPercentageVarianceExplained(self):
    """
    Return a handle to a new figure that shows the percentage of variance
    that each eigenvector explains.  This is just a percentage of the sum
    of the eigenvalues. The Row PCA view already holds the variances, which
    sum to 1, so inside this function they are simply converted to percentages.
    """
    f = pylab.figure()
    pylab.clf()
    eigenvalues = self.rowPCAView._RowPCAView__variances
    percentages = eigenvalues * 100

    cnos = range(1,self.nCols+1)
    
    pylab.plot(cnos,percentages,'b-o')
    pylab.xlim(0,self.nCols+1)
    pylab.xticks(cnos)
    pylab.grid('on')

    pylab.title('Variance Explained by Each Principal Component')
    pylab.xlabel('Principal Component Number')
    pylab.ylabel('Variance (percentage)')

    return f

  def plotPCNEigenvectorInOriginalColumnOrder(self, pcNum,
                                              conditionLabels=None):
    """
    Return a handle to a new figure that shows a trajectory plot for the
    PC-<N> eigenvector.  The x-axis is ordered as the columns in the
    dataset are... original column order.

    You may optionally provide conditionLabels, a list of strings that
    will be used as the x tick labels; by default, the x-axis will be
    labeled with a simple range of integers.
    """
    f = pylab.figure()
    pylab.clf()
    eigenvector = self.rowPCAView.matrix[pcNum-1,:]
    cnos = range(1,self.nCols+1)
    
#    if conditionLabels is not None:
#      pylab.plot(cnos,eigenvector,'b-o')
#    else:
#      pylab.plot(cnos,eigenvector,'b-o')
    pylab.plot(cnos,eigenvector,'b-o')

    pylab.xlim(0,self.nCols+1)

    pylab.xlabel('%d Conditions in Original Order' % (self.nCols))
    pylab.ylabel('eigenvector magnitude')

    if conditionLabels is not None:
#      labels = copy.copy(conditionLabels)
#      labels.insert(0,'')
#      pylab.xticks(pylab.arange(self.nCols+1),labels)
      pylab.xticks(cnos,conditionLabels)
      
    return f

  def plotPCNOutlierRowsInOriginalColumnOrder(self, pcNum,
                                              conditionLabels=None):
    """
    Return a handle to a new figure that shows a trajectory plot for the
    high and low outliers.  The x-axis is ordered as the columns in the
    dataset are... original column order.

    You may optionally provide conditionLabels, a list of strings that
    will be used as the x tick labels; by default, the x-axis will be
    labeled with a simple range of integers.  The list must be the same
    length as the conditions/dimensions.
    """
    if conditionLabels is not None:
      if len(conditionLabels) != self.dataset.numCols:
        msg = 'ERROR: conditionLabels list must be same size as data columns'
        raise ValueError, msg

    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    data = self.dataset.getData()
    cnos = range(1,self.nCols+1)
    
    highdata = pylab.zeros((len(highrows),self.dataset.numCols),'d')
    for i in range(0,len(highrows)):
      highdata[i,:] = data[highrows[i],:]
    
    lowdata = pylab.zeros((len(lowrows),self.dataset.numCols),'d')
    for i in range(0,len(lowrows)):
      lowdata[i,:] = data[lowrows[i],:]
    
    f = pylab.figure()
    pylab.clf()

#    if conditionLabels is not None:
#      for i in range(0,len(highrows)):
#        pylab.plot(cnos, highdata[i,:],'r-o')
#      for i in range(0,len(lowrows)):
#        pylab.plot(cnos, lowdata[i,:],'b-o')
#    else:
#      for i in range(0,len(highrows)):
#        pylab.plot(highdata[i,:],'r-o')
#      for i in range(0,len(lowrows)):
#        pylab.plot(lowdata[i,:],'b-o')
    for i in range(0,len(highrows)):
      pylab.plot(cnos, highdata[i,:],'r-')
    for i in range(0,len(lowrows)):
      pylab.plot(cnos, lowdata[i,:],'b-')

    pylab.xlim(0,self.nCols+1)

    pylab.xlabel('%d Conditions in Original Order' % (self.nCols))
    pylab.ylabel('Expression, log2(signal)')

    if conditionLabels is not None:
#      labels = copy.copy(conditionLabels)
#      labels.insert(0,'')
#      pylab.xticks(pylab.arange(self.nCols+1),labels)
      pylab.xticks(cnos,conditionLabels)
      
    return f


  def plotPCNOutlierRowsInSigGroupOrder(self, pcNum, conditionLabels=None):
    """
    Create and return a matlplotlib figure containing a plot of the
    outlier trajectories across conditions/dimensions where the
    conditions are reordered based (approximately) on significance of
    high vs. low.

    You may optionally provide conditionLabels, a list of strings that
    will be used as the x tick labels; by default, the x-axis will be
    labeled with a simple range of integers.  The list must be the same
    length as the conditions/dimensions.

    Note: Presently we order conditions/dimensions by mean diff, which is
    approximately ordered by significance of difference, but we SHOULD
    make this more precisely partitioned first into Up/Flat/Down, and
    then within group ordered by mean difference.  This ordering needs
    to correspond to the order of rows output by getOutputForSigGroups.
    """
    if conditionLabels is not None:
      if len(conditionLabels) != self.dataset.nCols:
        msg = 'ERROR: conditionLabels list must be same size as data columns'
        raise ValueError, msg

    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNum, self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    c = self.dataset.getLabeling('PC-%d %d-outlier Up/Flat/Down Columns' \
                                 % (pcNum, self.nOutliers))
    upcols   = c.getColsByLabel('up')
    flatcols = c.getColsByLabel('flat')
    downcols = c.getColsByLabel('down')

    data = self.dataset.getData()
    cnos = range(1,self.nCols+1)
    
    highdata = pylab.zeros((len(highrows),self.nCols),'d')
    for i in range(0,len(highrows)):
      highdata[i,:] = data[highrows[i],:]
    
    lowdata = pylab.zeros((len(lowrows),self.nCols),'d')
    for i in range(0,len(lowrows)):
      lowdata[i,:] = data[lowrows[i],:]
    
    # Average down columns, to get average for each condition
    lowMeans  = nx.average(lowdata, 0)
    highMeans = nx.average(highdata,0)
    
    meanDiffs = nx.subtract(highMeans,lowMeans);
    
#   Don't need this any more because we want either condition labels or
#   simplest auto-generated numeric labels, not explicit numeric labels
#   if conditionLabels is None:
#     conditionLabels = pylab.arange(1,self.nCols+1)

    meansort = pylab.sort(meanDiffs,0)
    # want to plot in descending order by mean differences
    meansort = pylab.fliplr([pylab.sort(meansort)])[0]
    lowsort  = pylab.zeros((len(lowrows), self.nCols),'d')
    highsort = pylab.zeros((len(highrows),self.nCols),'d')

    sortedLabels = []
    for i in range(0,self.nCols):
      currcond = pylab.find(meanDiffs == meansort[i])
      index = currcond[0]
      lowsort[:,i]  = lowdata[:,index]
      highsort[:,i] = highdata[:,index]
      if conditionLabels is not None:
        sortedLabels.append(conditionLabels[index])
    
    f = pylab.figure()
    pylab.clf()

## used to plot cnos, but probably don't need them to be explicit
##    for i in range(0,len(highrows)):
##      pylab.plot(cnos, highsort[i,:],'r-o')
##    for i in range(0,len(lowrows)):
##      pylab.plot(cnos, lowsort[i,:],'b-o')
## Now we plot without cnos whether labels will be added or not
#    for i in range(0,len(highrows)):
#      pylab.plot(highsort[i,:],'r-o')
#    for i in range(0,len(lowrows)):
#      pylab.plot(lowsort[i,:],'b-o')

# No, in fact, you NEED to make the x axis explicit for small sets to
# the x values range from 1->numcols rather than 0->numcols-1
# And we do this whether we are adding labels or not
    for i in range(0,len(highrows)):
      pylab.plot(cnos, highsort[i,:],'r-')
    for i in range(0,len(lowrows)):
      pylab.plot(cnos, lowsort[i,:],'b-')

#    upFlatDivide   = len(upcols)+0.5
#    flatDownDivide = len(upcols)+len(flatcols)+0.5
#    pylab.plot([upFlatDivide,upFlatDivide],
#               [self.minOriginalData,self.maxOriginalData],'g-')
#    pylab.plot([flatDownDivide,flatDownDivide],
#               [self.minOriginalData,self.maxOriginalData],'g-')

# FIXME: this is matplotlib .7x feature
#    pylab.xlim(0,self.nCols+1)

    pylab.xlim(0,self.nCols+1)

    pylab.xlabel('%d Conditions Ordered by Mean Difference' % (self.nCols))
    pylab.ylabel('Expression, log2(signal)')
    
    if len(sortedLabels)>0:
#      sortedLabels.insert(0,'')
      pylab.xticks(cnos,sortedLabels)
#      pylab.xticks(pylab.arange(self.nCols+1),sortedLabels)

#    show()
    
    return f

  def plotPCvsPCWithOutliersInY(self, pcNumForXAxis, pcNumForYAxis):
    """
    Create and return a matplotlib figure containing a scatter plot of
    data points in the dataset when projected onto one principal component
    versus another.  PC numbers passed in need to be 1-origin. 
    """
    l = self.dataset.getLabeling('PC-%d %d High/Low' % (pcNumForYAxis,
                                                        self.nOutliers))
    highrows = l.getRowsByLabel('high')
    lowrows = l.getRowsByLabel('low')

    f = pylab.figure()
    pylab.clf()

    pcaData = self.rowPCAView.getData()
    pylab.plot(pcaData[:,pcNumForXAxis-1],pcaData[:,pcNumForYAxis-1],'b.')

    for highOutlier in highrows:
      pylab.plot([pcaData[highOutlier,pcNumForXAxis-1]],
                 [pcaData[highOutlier,pcNumForYAxis-1]],'ro')

    for lowOutlier in lowrows:
      pylab.plot([pcaData[lowOutlier,pcNumForXAxis-1]],
                 [pcaData[lowOutlier,pcNumForYAxis-1]],'ro')

    pylab.title('PC %d: %d Highest and %d Lowest Outliers' % (pcNumForYAxis,
                                                              self.nOutliers,
                                                              self.nOutliers))
    percentages = self.rowPCAView._RowPCAView__variances * 100;
    pylab.xlabel('PC %d (%4.2f%%)' %(pcNumForXAxis,percentages[pcNumForXAxis-1]))
    pylab.ylabel('PC %d (%4.2f%%)' %(pcNumForYAxis,percentages[pcNumForYAxis-1]))

#    pylab.show()

    return f


  def generateResults(self,rowLabelingNames,colLabelingNames,
                      conditionLabels=None):
    """
    For each principal component, call the major result generating
    functions and save those results to appropriately-named files.
    """
    f = self.plotPercentageVarianceExplained()
    pylab.savefig('percentage-variance-explained')
      
    # Within this function pcNum is 1-origin to match how users would call 
    # the various user-level functions
    for pcNum in range(1,self.maxPCNum+1):

      if self.verbose:
        print '  Writing results for PC %d' % pcNum

      if pcNum > 1:
        f = self.plotPCvsPCWithOutliersInY(pcNum-1, pcNum)
        pylab.savefig('pc%02d-outliers' % pcNum)

      f = self.plotPCNEigenvectorInOriginalColumnOrder(pcNum,conditionLabels)
      pylab.savefig('pc%02d-eigenvector' % pcNum)

      f = self.plotPCNOutlierRowsInOriginalColumnOrder(pcNum,conditionLabels)
      pylab.savefig('pc%02d-outlier-trajectories-order-original' % pcNum)

      f = self.plotPCNOutlierRowsInSigGroupOrder(pcNum,conditionLabels)
      pylab.savefig('pc%02d-outlier-trajectories-order-meandiff' % pcNum)

      output = self.getOutputForPCNOutliers(pcNum,rowLabelingNames)
      write2DStringArrayToFile(output, 'pc%02d-outliers.txt' % pcNum)

      output = self.getOutputForSigGroups(pcNum,colLabelingNames)
      write2DStringArrayToFile(output, 'pc%02d-condition-groups.txt' % pcNum)


def write2DStringArrayToFile(stringArray, filename, delim='\t'):
  """
  Simple utility function to spew a 2D string array to a tab-delimited
  text file.
  """
  fd = open(filename,'w')
  for row in stringArray:
    for col in row:
      fd.write('%s%s' %(col,delim))
    fd.write('\n')
  fd.close()
    

def secondColumnCompare(val1,val2):
  """
  Used by getOutputForPCNOutliers to sort the output by the second column
  value, in descending order.
  """
  if val1[1] > val2[1]:
    return -1
  else:
    return 1
